import 'package:chopper/chopper.dart';
import 'package:http/http.dart' as http;
import 'package:adeptarch/src/api/backend_api.dart' as BackendAPI;
import 'package:adeptutils/adeptutils.dart';
import 'dart:async';

/// Created by krrigan on 02/27/19.

abstract class APIService<I extends ChopperService, DataType extends Jsonable, ErrorType> implements ChopperService
{
    I service;
    DataType dataType;
    ErrorType errorType;

    APIService(I service, {this.dataType, this.errorType, BackendAPI.BackendAPI backendAPI})
    {
      this.service = service;
      if(backendAPI != null) backendAPI.addService(service); else
          BackendAPI.backendAPI.addService(service);
    }

    dynamic getAuthorization();

    Future<Response> getSaveResponse(Future<Response> futureResponse) async{
      if(futureResponse == null) return null;
      Response response;
      try{
          response = await futureResponse;
      } catch(e){
          if(e is Response) response = e; else throw e;
      }
      return response;
    }

    Future<Response<DataType>> parseResponse(Future<Response> futureResponse) async{
      return genericParseResponse(futureResponse, dataType);
    }

    Future<Response<List<DataType>>> parseResponseAsList(Future<Response> futureResponse) async{
      return genericParseResponseAsList(futureResponse, dataType);
    }

    Object _parseError(Response response) {
      Object errorTypeResult = response?.error;
      try{
        if (errorType is Jsonable)
          errorTypeResult = response?.error is String ? (errorType as Jsonable).fromJsonString(response?.error) : (errorType as Jsonable).fromJsonMap(response?.error);
      }catch(e){
        print(e);
      }
      return errorTypeResult;
    }

    Future<Response<DataType>> genericParseResponse<DataType extends Jsonable>(Future<Response> futureResponse, DataType dataType) async{
      Response response = await getSaveResponse(futureResponse);
      try{
        if(dataType != null){
          DataType dataTypeResult = response.body is String ? dataType.fromJsonString(response.body) : dataType.fromJsonMap(response.body);
          Object errorTypeResult = _parseError(response);
          return Response<DataType>(response.base, dataTypeResult, error: errorTypeResult);
        }
      }catch(e){
        String message = e.toString();
        response = Response(
            http.Response(
              response?.body?.toString(),
              Jsonable.jsonParserError,
              headers: response?.base?.headers,
              isRedirect: response?.base?.isRedirect,
              persistentConnection: response?.base?.persistentConnection,
              reasonPhrase: response?.base?.reasonPhrase,
              request: response?.base?.request,
            ),
            response.body,
            error: message
        );
        print(e);
      }
      return Response<DataType>(response.base, null, error: response.error);
    }

    Future<Response<List<DataType>>> genericParseResponseAsList<DataType extends Jsonable>(Future<Response> futureResponse, DataType dataType) async{
      Response response = await getSaveResponse(futureResponse);
      try{
        if(dataType != null){
          List<DataType> dataList = response.body is String ? dataType.fromJsonStringList(response.body) : dataType.fromJsonList(response.body);
          Object errorTypeResult = _parseError(response);
          return Response<List<DataType>>(response.base, dataList, error: errorTypeResult);
        }
      }catch(e){
        String message = e.toString();
        response = Response(
          http.Response(
            response?.body?.toString(),
            Jsonable.jsonParserError,
            headers: response?.base?.headers,
            isRedirect: response?.base?.isRedirect,
            persistentConnection: response?.base?.persistentConnection,
            reasonPhrase: response?.base?.reasonPhrase,
            request: response?.base?.request,
          ),
          response.body,
          error: message
        );
        print(e);
      }
      return Response<List<DataType>>(response.base, null, error: response.error);
    }

    @override
    void dispose() => service.dispose();

    @override
    Type get definitionType => service.definitionType;

    @override
    ChopperClient get client => service.client;

    @override
    set client(ChopperClient client) => service.client = client;
}
