import 'package:adeptarch/src/widgets/widgets.dart';
import 'package:adeptarch/src/filters/filters.dart';
import 'package:flutter/material.dart';

class BoolFilterView extends BaseFilterView<BoolFilter> {

  String textUnspecified, textTrue, textFalse;

  BoolFilterView({
    FilterController<BoolFilter> filterController,
    String hint,
    this.textUnspecified,
    this.textTrue,
    this.textFalse,
  }) : super(
    filterController: filterController,
    hint: hint,
  ){
    textUnspecified ??= "Unspecified";
    textTrue ??= "Yes";
    textFalse ??= "No";
  }

  @override
  State createState()  => _BoolFilterViewState();
}

class _BoolFilterViewState extends State<BoolFilterView>{
  Color colorPrimary, colorTextTertiary;
  List<DropdownMenuItem<bool>> boolFilterValues;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    generateFilterItems();
    onFilterChanged();
  }

  void generateFilterItems(){
    boolFilterValues = [
      DropdownMenuItem(
        value: null,
        child: Text(widget.textUnspecified),
      ),
      DropdownMenuItem(
        value: true,
        child: Text(widget.textTrue),
      ),
      DropdownMenuItem(
        value: false,
        child: Text(widget.textFalse),
      ),
    ];
  }

  void listeners() {
    widget.filterController.addListener(onFilterChanged);
  }

  void onFilterChanged(){
    BoolFilter filter = widget.filterController.value;
    if(filter != null){
    }
  }

  void checkColors(context){
    colorPrimary ??= Theme.of(context).primaryColor;
    colorTextTertiary ??= Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return Container(
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Text(
            widget.hint,
            style: TextStyle(color: widget.filterController.value.value != null ? colorPrimary : colorTextTertiary, fontSize: 12),
          ),
          Theme(
            data: ThemeData.light(),
            child: DropdownButton<bool>(
              isExpanded: true,
              items: boolFilterValues,
              value: widget.filterController.value.value,
              onChanged: (value){
                if(value == null) {
                  widget.filterController.reset();
                } else{
                  widget.filterController.value.equals(value);
                  widget.filterController.clearError();
                }
                setState(() {});
              },
            ),
          ),
        ],
      ),
    );
  }

  @override
  void dispose() {
    widget.filterController.removeListener(onFilterChanged);
    super.dispose();
  }
}